-module(ai_core).

-export([
    init/1,
    make_decision/2,
    evaluate_hand/1,
    predict_plays/2,
    update_knowledge/2,
    calculate_win_rate/1
]).

-include("card_types.hrl").
-import(game_logic, [validate_play/2, analyze_card_pattern/1, calculate_card_value/1, evaluate_hand_strength/1, find_singles/1]).

-record(ai_state, {
    role,                % dizhu | nongmin
    hand_cards = [],     % 当前手牌
    known_cards = [],    % 已知的牌
    played_cards = [],   % 已打出的牌
    player_history = [], % 玩家出牌历史
    game_stage,         % early_game | mid_game | end_game
    strategy_cache = #{} % 策略缓存
}).

-record(play_context, {
    last_play,          % 上一手牌
    cards_remaining,    % 剩余手牌数
    num_greater_cards,  % 比上家大的牌数量
    control_factor,     % 控制因子
    must_play = false,   % 是否必须出牌
    game_stage = early_game % 游戏进度
}).

%% 初始化AI状态
init(Role) ->
    #ai_state{
        role = Role,
        game_stage = early_game
    }.

%% 做出决策
make_decision(AIState, GameState) ->
    Context = analyze_context(AIState, GameState),
    case should_play(Context, AIState) of
        true ->
            select_best_play(Context, AIState);
        false ->
            {pass, AIState}
    end.

%% 评估手牌
evaluate_hand(Cards) ->
    Components = analyze_components(Cards),
    calculate_hand_value(Components).

%% 预测可能的出牌
predict_plays(Cards, LastPlay) ->
    ValidPlays = generate_valid_plays(Cards, LastPlay),
    score_potential_plays(ValidPlays, LastPlay).

%% 更新知识库
update_knowledge(AIState, Event) ->
    update_state_with_event(AIState, Event).

%% 计算胜率
calculate_win_rate(AIState) ->
    HandStrength = evaluate_hand(AIState#ai_state.hand_cards),
    PositionValue = evaluate_position(AIState),
    ControlValue = evaluate_control(AIState),

    calculate_probability(HandStrength, PositionValue, ControlValue).

%% 内部函数

%% 分析上下文
analyze_context(AIState, GameState) ->
    LastPlay = get_last_play(GameState),
    CardsRemaining = length(AIState#ai_state.hand_cards),
    GreaterCards = count_greater_cards(AIState#ai_state.hand_cards, LastPlay),
    ControlFactor = calculate_control_factor(AIState, GameState),

    #play_context{
        last_play = LastPlay,
        cards_remaining = CardsRemaining,
        num_greater_cards = GreaterCards,
        control_factor = ControlFactor,
        must_play = must_play(AIState, GameState)
    }.

%% 判断是否应该出牌
should_play(Context, AIState) ->
    case Context#play_context.must_play of
        true -> true;
        false ->
            case Context#play_context.last_play of
                none -> true;
                _ ->
                    should_beat_last_play(Context, AIState)
            end
    end.

%% 选择最佳出牌
select_best_play(Context, AIState) ->
    Candidates = generate_candidates(AIState#ai_state.hand_cards, Context),
    ScoredPlays = [
        {score_play(Play, Context, AIState), Play}
        || Play <- Candidates
    ],
    select_highest_scored_play(ScoredPlays, AIState).

%% 分析牌型组件
analyze_components(Cards) ->
    GroupedCards = group_cards_by_value(Cards),
    #{
        singles => game_logic:find_singles(GroupedCards),
        pairs => game_logic:find_pairs(GroupedCards),
        triples => game_logic:find_triples(GroupedCards),
        sequences => find_sequences(GroupedCards),
        bombs => game_logic:find_bombs(GroupedCards)
    }.

%% 计算手牌价值
calculate_hand_value(Components) ->
    SinglesValue = calculate_singles_value(maps:get(singles, Components, [])),
    PairsValue = calculate_pairs_value(maps:get(pairs, Components, [])),
    TriplesValue = calculate_triples_value(maps:get(triples, Components, [])),
    SequencesValue = calculate_sequences_value(maps:get(sequences, Components, [])),
    BombsValue = calculate_bombs_value(maps:get(bombs, Components, [])),

    SinglesValue + PairsValue + TriplesValue + SequencesValue + BombsValue.

%% 生成有效出牌选择
generate_valid_plays(Cards, LastPlay) ->
    case LastPlay of
        none ->
            generate_leading_plays(Cards);
        Play ->
            generate_following_plays(Cards, Play)
    end.

%% 评分潜在出牌
score_potential_plays(Plays, LastPlay) ->
    [{Play, score_play_potential(Play, LastPlay)} || Play <- Plays].

%% 更新状态
update_state_with_event(AIState, {play_cards, Player, Cards}) ->
    NewPlayed = AIState#ai_state.played_cards ++ Cards,
    NewHistory = [{Player, Cards} | AIState#ai_state.player_history],
    AIState#ai_state{
        played_cards = NewPlayed,
        player_history = NewHistory
    };
update_state_with_event(AIState, {game_stage, NewStage}) ->
    AIState#ai_state{game_stage = NewStage};
update_state_with_event(AIState, _) ->
    AIState.

%% 计算胜率概率
calculate_probability(HandStrength, PositionValue, ControlValue) ->
    BaseProb = HandStrength * 0.5 + PositionValue * 0.3 + ControlValue * 0.2,
    normalize_probability(BaseProb).

%% 计算控制因子
calculate_control_factor(AIState, GameState) ->
    ControlCards = count_control_cards(AIState#ai_state.hand_cards),
    TotalCards = count_total_remaining_cards(GameState),
    RemainingCards = length(AIState#ai_state.hand_cards),

    ControlRatio = ControlCards / max(1, RemainingCards),
    PositionBonus = calculate_position_bonus(AIState, GameState),

    ControlRatio * PositionBonus.

%% 判断是否必须出牌
must_play(AIState, GameState) ->
    is_current_player(AIState, GameState) andalso
        not has_active_play(GameState).

%% 评估是否应该大过上家
should_beat_last_play(Context, AIState) ->
    case Context#play_context.last_play of
        none -> true;
        LastPlay ->
            HandStrength = evaluate_hand(AIState#ai_state.hand_cards),
            ControlLevel = Context#play_context.control_factor,
            CardsLeft = Context#play_context.cards_remaining,

            should_beat(HandStrength, ControlLevel, CardsLeft, LastPlay)
    end.

%% 生成候选出牌
generate_candidates(Cards, Context) ->
    BasePlays = case Context#play_context.last_play of
        none -> generate_leading_plays(Cards);
        LastPlay -> generate_following_plays(Cards, LastPlay)
    end,

    filter_candidates(BasePlays, Context).

%% 对出牌进行评分
score_play(Play, Context, AIState) ->
    BaseScore = calculate_base_score(Play),
    TempoScore = calculate_tempo_score(Play, Context),
    ControlScore = calculate_control_score(Play, Context, AIState),
    EfficiencyScore = calculate_efficiency_score(Play, Context),

    FinalScore = BaseScore * 0.4 +
        TempoScore * 0.2 +
        ControlScore * 0.3 +
        EfficiencyScore * 0.1,

    adjust_score_for_context(FinalScore, Play, Context, AIState).

%% 选择得分最高的出牌
select_highest_scored_play(ScoredPlays, AIState) ->
    case lists:sort(fun({Score1, _}, {Score2, _}) ->
        Score1 >= Score2
    end, ScoredPlays) of
        [{Score, Play}|_] when Score > 0 ->
            {play, Play, update_after_play(Play, AIState)};
        _ ->
            {pass, AIState}
    end.

%% 计算单牌价值
calculate_singles_value(Singles) ->
    lists:sum([
        case Value of
            V when V >= ?CARD_VALUE_2 -> Value * 1.5;
            _ -> Value
        end || {Value, _} <- Singles
    ]).

%% 计算对子价值
calculate_pairs_value(Pairs) ->
    lists:sum([Value * 2.2 || {Value, _} <- Pairs]).

%% 计算三张价值
calculate_triples_value(Triples) ->
    lists:sum([Value * 3.5 || {Value, _} <- Triples]).

%% 计算顺子价值
calculate_sequences_value(Sequences) ->
    lists:sum([
        Value * length(Cards) * 1.8
        || {Value, Cards} <- Sequences
    ]).

%% 计算炸弹价值
calculate_bombs_value(Bombs) ->
    lists:sum([Value * 10.0 || {Value, _} <- Bombs]).

%% 生成首出牌型
generate_leading_plays(Cards) ->
    Components = analyze_components(Cards),
    Singles = generate_single_plays(Components),
    Pairs = generate_pair_plays(Components),
    Triples = generate_triple_plays(Components),
    Sequences = generate_sequence_plays(Components),
    Bombs = generate_bomb_plays(Components),

    Singles ++ Pairs ++ Triples ++ Sequences ++ Bombs.

%% 生成跟牌选择
generate_following_plays(Cards, {Type, Value, _} = LastPlay) ->
    Components = analyze_components(Cards),
    ValidPlays = case Type of
        ?CARD_TYPE_SINGLE -> find_greater_singles(Components, Value);
        ?CARD_TYPE_PAIR -> find_greater_pairs(Components, Value);
        ?CARD_TYPE_THREE -> find_greater_triples(Components, Value);
        ?CARD_TYPE_THREE_ONE -> find_greater_three_one(Components, Value);
        ?CARD_TYPE_THREE_TWO -> find_greater_three_two(Components, Value);
        ?CARD_TYPE_STRAIGHT -> find_greater_straight(Components, Value);
        ?CARD_TYPE_STRAIGHT_PAIR -> find_greater_straight_pair(Components, Value);
        ?CARD_TYPE_PLANE -> find_greater_plane(Components, Value);
        ?CARD_TYPE_BOMB -> find_greater_bomb(Components, Value);
        _ -> []
    end,
    BombPlays = find_bomb_plays(Cards),
    RocketPlay = find_rocket_play(Cards),

    ValidPlays ++ BombPlays ++ RocketPlay.

%% 计算基础分数
calculate_base_score({Type, Value, Cards}) ->
    BaseValue = Value * length(Cards),
    TypeMultiplier = case Type of
        ?CARD_TYPE_ROCKET -> 100.0;
        ?CARD_TYPE_BOMB -> 80.0;
        ?CARD_TYPE_STRAIGHT -> 40.0;
        ?CARD_TYPE_STRAIGHT_PAIR -> 35.0;
        ?CARD_TYPE_PLANE -> 30.0;
        ?CARD_TYPE_THREE_TWO -> 25.0;
        ?CARD_TYPE_THREE_ONE -> 20.0;
        ?CARD_TYPE_THREE -> 15.0;
        ?CARD_TYPE_PAIR -> 10.0;
        ?CARD_TYPE_SINGLE -> 5.0
    end,
    BaseValue * TypeMultiplier / 100.0.

%% 计算节奏分数
calculate_tempo_score(Play, Context) ->
    case Context#play_context.game_stage of
        early_game -> calculate_early_tempo(Play, Context);
        mid_game -> calculate_mid_tempo(Play, Context);
        end_game -> calculate_end_tempo(Play, Context)
    end.

%% 计算控制分数
calculate_control_score(Play, Context, AIState) ->
    {Type, Value, _} = Play,
    BaseControl = case Type of
        ?CARD_TYPE_BOMB -> 1.0;
        ?CARD_TYPE_ROCKET -> 1.0;
        _ when Value >= ?CARD_VALUE_2 -> 0.8;
        _ -> 0.5
    end,
    BaseControl * Context#play_context.control_factor.

%% 计算效率分数
calculate_efficiency_score(Play, Context) ->
    {_, _, Cards} = Play,
    CardsUsed = length(Cards),
    RemainingCards = Context#play_context.cards_remaining - CardsUsed,
    Efficiency = CardsUsed / max(1, Context#play_context.cards_remaining),
    Efficiency * (1 + (20 - RemainingCards) / 20).

%% 根据上下文调整分数
adjust_score_for_context(Score, Play, Context, AIState) ->
    RoleMultiplier = case AIState#ai_state.role of
        dizhu -> 1.2;
        nongmin -> 1.0
    end,

    StageMultiplier = case Context#play_context.game_stage of
        early_game -> 0.9;
        mid_game -> 1.0;
        end_game -> 1.1
    end,

    Score * RoleMultiplier * StageMultiplier.

%% 更新出牌后的状态
update_after_play(Play, AIState) ->
    {_, _, Cards} = Play,
    NewHand = AIState#ai_state.hand_cards -- Cards,
    NewPlayed = AIState#ai_state.played_cards ++ Cards,

    AIState#ai_state{
        hand_cards = NewHand,
        played_cards = NewPlayed
    }.

%% 辅助函数

normalize_probability(P) ->
    max(0.0, min(1.0, P)).

count_control_cards(Cards) ->
    length([C || {V, _} = C <- Cards, V >= ?CARD_VALUE_2]).

calculate_position_bonus(AIState, GameState) ->
    case get_position(AIState, GameState) of
        first -> 1.2;
        middle -> 1.0;
        last -> 0.8
    end.

get_position(AIState, GameState) ->
    % 根据游戏状态判断位置
    first.  % 简化实现，实际需要根据具体游戏状态判断

is_current_player(AIState, GameState) ->
    % 判断是否当前玩家
    true.  % 简化实现，实际需要根据具体游戏状态判断

has_active_play(GameState) ->
    % 判断是否有活跃的出牌
    false.  % 简化实现，实际需要根据具体游戏状态判断

should_beat(HandStrength, ControlLevel, CardsLeft, LastPlay) ->
    BaseThreshold = 0.6,
    StrengthFactor = HandStrength / 100,
    ControlFactor = ControlLevel / 100,
    CardsFactor = (20 - CardsLeft) / 20,

    PlayThreshold = BaseThreshold * (StrengthFactor + ControlFactor + CardsFactor) / 3,
    evaluate_play_value(LastPlay) < PlayThreshold.

evaluate_play_value({Type, Value, Cards}) ->
    BaseValue = case Type of
        ?CARD_TYPE_ROCKET -> 1.0;
        ?CARD_TYPE_BOMB -> 0.9;
        ?CARD_TYPE_PLANE -> 0.7;
        ?CARD_TYPE_STRAIGHT -> 0.6;
        ?CARD_TYPE_STRAIGHT_PAIR -> 0.5;
        ?CARD_TYPE_THREE_TWO -> 0.4;
        ?CARD_TYPE_THREE_ONE -> 0.3;
        ?CARD_TYPE_THREE -> 0.25;
        ?CARD_TYPE_PAIR -> 0.2;
        ?CARD_TYPE_SINGLE -> 0.1
    end,

    ValueBonus = Value / ?CARD_VALUE_BIG_JOKER,
    CardCountFactor = length(Cards) / 10,

    BaseValue * (1 + ValueBonus) * (1 + CardCountFactor).

%% 牌型生成函数
generate_single_plays(Components) ->
    [{?CARD_TYPE_SINGLE, Value, [Card]} ||
        {Value, Card} <- maps:get(singles, Components, [])].

generate_pair_plays(Components) ->
    [{?CARD_TYPE_PAIR, Value, Cards} ||
        {Value, Cards} <- maps:get(pairs, Components, [])].

generate_triple_plays(Components) ->
    Triples = maps:get(triples, Components, []),
    BasicTriples = [{?CARD_TYPE_THREE, Value, Cards} ||
        {Value, Cards} <- Triples],
    % 生成三带一和三带二
    generate_triple_combinations(Triples, Components).

generate_sequence_plays(Components) ->
    Sequences = maps:get(sequences, Components, []),
    [{?CARD_TYPE_STRAIGHT, Value, Cards} ||
        {Value, Cards} <- Sequences].

generate_bomb_plays(Components) ->
    [{?CARD_TYPE_BOMB, Value, Cards} ||
        {Value, Cards} <- maps:get(bombs, Components, [])].

generate_triple_combinations(Triples, Components) ->
    Singles = maps:get(singles, Components, []),
    Pairs = maps:get(pairs, Components, []),

    ThreeOne = generate_three_one(Triples, Singles),
    ThreeTwo = generate_three_two(Triples, Pairs),

    ThreeOne ++ ThreeTwo.

generate_three_one(Triples, Singles) ->
    [{?CARD_TYPE_THREE_ONE, TripleValue, TripleCards ++ [SingleCard]} ||
        {TripleValue, TripleCards} <- Triples,
        {SingleValue, SingleCard} <- Singles,
        SingleValue =/= TripleValue].

generate_three_two(Triples, Pairs) ->
    [{?CARD_TYPE_THREE_TWO, TripleValue, TripleCards ++ PairCards} ||
        {TripleValue, TripleCards} <- Triples,
        {PairValue, PairCards} <- Pairs,
        PairValue =/= TripleValue].

%% 评估出牌潜力
score_play_potential(Play, LastPlay) ->
    {Type, Value, Cards} = Play,
    BaseScore = case Type of
        ?CARD_TYPE_ROCKET -> 1.0;
        ?CARD_TYPE_BOMB -> 0.9;
        ?CARD_TYPE_STRAIGHT -> 0.8;
        ?CARD_TYPE_STRAIGHT_PAIR -> 0.75;
        ?CARD_TYPE_PLANE -> 0.7;
        ?CARD_TYPE_THREE_TWO -> 0.6;
        ?CARD_TYPE_THREE_ONE -> 0.5;
        ?CARD_TYPE_THREE -> 0.4;
        ?CARD_TYPE_PAIR -> 0.3;
        ?CARD_TYPE_SINGLE -> 0.2
    end,
    
    % 根据牌值调整分数
    ValueFactor = Value / ?CARD_VALUE_BIG_JOKER,
    
    % 根据出牌数量调整分数
    CountFactor = length(Cards) / 10,
    
    % 计算最终分数
    BaseScore * (1 + ValueFactor) * (1 + CountFactor).

%% 内部辅助函数

%% 按值分组牌
group_cards_by_value(Cards) ->
    lists:foldl(fun(Card, Acc) ->
        {Value, _} = Card,
        maps:update_with(Value, fun(List) -> [Card | List] end, [Card], Acc)
    end, #{}, Cards).

%% 查找顺子
find_sequences(GroupedCards) ->
    Values = lists:sort(maps:keys(GroupedCards)),
    find_sequences(Values, GroupedCards, []).

find_sequences([], _, Acc) -> Acc;
find_sequences([V | Rest], GroupedCards, Acc) ->
    case find_sequence_starting_at(V, Rest, GroupedCards) of
        {Seq, NewRest} when length(Seq) >= 5 ->
            Cards = lists:flatten([maps:get(Value, GroupedCards) || Value <- Seq]),
            find_sequences(NewRest, GroupedCards, [{V, Cards} | Acc]);
        {_, NewRest} ->
            find_sequences(NewRest, GroupedCards, Acc)
    end.

find_sequence_starting_at(Start, Rest, _) when Start >= ?CARD_VALUE_2 ->
    {[], Rest}; % 2及以上不能组成顺子
find_sequence_starting_at(Start, Rest, GroupedCards) ->
    find_sequence_starting_at(Start, Rest, GroupedCards, [Start]).

find_sequence_starting_at(_, [], _, Acc) ->
    {lists:reverse(Acc), []};
find_sequence_starting_at(Prev, [Next | Rest], GroupedCards, Acc) ->
    case Next of
        Next when Next =:= Prev + 1, Next < ?CARD_VALUE_2 ->
            find_sequence_starting_at(Next, Rest, GroupedCards, [Next | Acc]);
        _ ->
            {lists:reverse(Acc), [Next | Rest]}
    end.

%% 过滤候选出牌
filter_candidates(Plays, Context) ->
    % 根据上下文过滤出牌
    % 例如：在结束阶段可能会优先选择能一次出完的牌
    case Context#play_context.game_stage of
        end_game when Context#play_context.cards_remaining =< 4 ->
            % 在结束阶段且剩余牌数少时，优先考虑能一次出完的牌
            [Play || Play = {_, _, Cards} <- Plays, length(Cards) =:= Context#play_context.cards_remaining];
        _ ->
            Plays
    end.

%% 获取上一手牌
get_last_play(GameState) ->
    % 从游戏状态中获取上一手牌
    % 简化实现，实际需要根据具体游戏状态获取
    maps:get(last_play, GameState, none).

%% 计算比上家大的牌数量
count_greater_cards(Cards, LastPlay) ->
    case LastPlay of
        none -> length(Cards);
        {Type, Value, _} ->
            Components = analyze_components(Cards),
            length(find_greater_plays(Cards, Type, Value))
    end.

%% 评估位置
evaluate_position(AIState) ->
    % 评估当前位置的价值
    % 简化实现，实际需要考虑更多因素
    case AIState#ai_state.role of
        dizhu -> 0.7;
        nongmin -> 0.5
    end.

%% 评估控制力
evaluate_control(AIState) ->
    % 评估控制力
    % 简化实现，实际需要考虑更多因素
    ControlCards = count_control_cards(AIState#ai_state.hand_cards),
    TotalCards = length(AIState#ai_state.hand_cards),
    ControlCards / max(1, TotalCards).

%% 计算剩余总牌数
count_total_remaining_cards(GameState) ->
    % 计算游戏中剩余的总牌数
    % 简化实现，实际需要根据具体游戏状态计算
    maps:get(total_remaining, GameState, 54).

%% 查找特定牌型
find_greater_plays(Cards, Type, MinValue) ->
    Components = analyze_components(Cards),
    case Type of
        ?CARD_TYPE_SINGLE ->
            find_greater_singles(Components, MinValue);
        ?CARD_TYPE_PAIR ->
            find_greater_pairs(Components, MinValue);
        ?CARD_TYPE_THREE ->
            find_greater_triples(Components, MinValue);
        ?CARD_TYPE_THREE_ONE ->
            find_greater_three_one(Components, MinValue);
        ?CARD_TYPE_THREE_TWO ->
            find_greater_three_two(Components, MinValue);
        ?CARD_TYPE_STRAIGHT ->
            find_greater_straight(Components, MinValue);
        ?CARD_TYPE_STRAIGHT_PAIR ->
            find_greater_straight_pair(Components, MinValue);
        ?CARD_TYPE_PLANE ->
            find_greater_plane(Components, MinValue);
        ?CARD_TYPE_BOMB ->
            find_greater_bomb(Components, MinValue);
        _ -> []
    end.

find_greater_singles(Components, MinValue) ->
    [{?CARD_TYPE_SINGLE, Value, [Card]} ||
        {Value, Card} <- maps:get(singles, Components, []),
        Value > MinValue].

find_greater_pairs(Components, MinValue) ->
    [{?CARD_TYPE_PAIR, Value, Cards} ||
        {Value, Cards} <- maps:get(pairs, Components, []),
        Value > MinValue].

find_greater_triples(Components, MinValue) ->
    [{?CARD_TYPE_THREE, Value, Cards} ||
        {Value, Cards} <- maps:get(triples, Components, []),
        Value > MinValue].

find_greater_three_one(Components, MinValue) ->
    Triples = [{V, C} || {V, C} <- maps:get(triples, Components, []),
        V > MinValue],
    Singles = maps:get(singles, Components, []),
    generate_three_one(Triples, Singles).

find_greater_three_two(Components, MinValue) ->
    Triples = [{V, C} || {V, C} <- maps:get(triples, Components, []),
        V > MinValue],
    Pairs = maps:get(pairs, Components, []),
    generate_three_two(Triples, Pairs).

find_greater_straight(Components, MinValue) ->
    Sequences = maps:get(sequences, Components, []),
    [{?CARD_TYPE_STRAIGHT, Value, Cards} ||
        {Value, Cards} <- Sequences,
        Value > MinValue].

find_greater_straight_pair(Components, MinValue) ->
    Sequences = find_pair_sequences(Components),
    [{?CARD_TYPE_STRAIGHT_PAIR, Value, Cards} ||
        {Value, Cards} <- Sequences,
        Value > MinValue].

find_greater_plane(Components, MinValue) ->
    Planes = find_planes(Components),
    [{?CARD_TYPE_PLANE, Value, Cards} ||
        {Value, Cards} <- Planes,
        Value > MinValue].

find_greater_bomb(Components, MinValue) ->
    [{?CARD_TYPE_BOMB, Value, Cards} ||
        {Value, Cards} <- maps:get(bombs, Components, []),
        Value > MinValue].

find_bomb_plays(Cards) ->
    Components = analyze_components(Cards),
    [{?CARD_TYPE_BOMB, Value, Cards} ||
        {Value, Cards} <- maps:get(bombs, Components, [])].

find_rocket_play(Cards) ->
    Components = analyze_components(Cards),
    case find_rocket(Components) of
        {ok, Rocket} -> [Rocket];
        _ -> []
    end.

find_rocket(Components) ->
    case {find_card(?CARD_VALUE_SMALL_JOKER, Components),
        find_card(?CARD_VALUE_BIG_JOKER, Components)} of
        {{ok, Small}, {ok, Big}} ->
            {ok, {?CARD_TYPE_ROCKET, ?CARD_VALUE_BIG_JOKER, [Small, Big]}};
        _ ->
            false
    end.

find_card(Value, Components) ->
    Singles = maps:get(singles, Components, []),
    case lists:keyfind(Value, 1, Singles) of
        {Value, Card} -> {ok, Card};
        _ -> false
    end.

find_pair_sequences(Components) ->
    Pairs = maps:get(pairs, Components, []),
    find_consecutive_pairs(lists:sort(Pairs), []).

find_planes(Components) ->
    Triples = maps:get(triples, Components, []),
    find_consecutive_triples(lists:sort(Triples), []).

find_consecutive_pairs([], Acc) -> lists:reverse(Acc);
find_consecutive_pairs([{V1, Cards1} | Rest], Acc) ->
    case find_consecutive_pair_sequence(V1, Cards1, Rest) of
        {Sequence, NewRest} when length(Sequence) >= 3 ->
            find_consecutive_pairs(NewRest, [{V1, Sequence} | Acc]);
        _ ->
            find_consecutive_pairs(Rest, Acc)
    end.

find_consecutive_pair_sequence(Value, Cards, Rest) ->
    find_consecutive_pair_sequence(Value, Cards, Rest, [Cards]).

find_consecutive_pair_sequence(Value, _, [], Acc) ->
    {lists:flatten(lists:reverse(Acc)), []};
find_consecutive_pair_sequence(Value, _, [{NextValue, NextCards} | Rest], Acc)
    when NextValue =:= Value + 1 ->
    find_consecutive_pair_sequence(NextValue, NextCards, Rest, [NextCards | Acc]);
find_consecutive_pair_sequence(_, _, Rest, Acc) ->
    {lists:flatten(lists:reverse(Acc)), Rest}.

find_consecutive_triples([], Acc) -> lists:reverse(Acc);
find_consecutive_triples([{V1, Cards1} | Rest], Acc) ->
    case find_consecutive_triple_sequence(V1, Cards1, Rest) of
        {Sequence, NewRest} when length(Sequence) >= 6 ->
            find_consecutive_triples(NewRest, [{V1, Sequence} | Acc]);
        _ ->
            find_consecutive_triples(Rest, Acc)
    end.

find_consecutive_triple_sequence(Value, Cards, Rest) ->
    find_consecutive_triple_sequence(Value, Cards, Rest, [Cards]).

find_consecutive_triple_sequence(Value, _, [], Acc) ->
    {lists:flatten(lists:reverse(Acc)), []};
find_consecutive_triple_sequence(Value, _, [{NextValue, NextCards} | Rest], Acc)
    when NextValue =:= Value + 1 ->
    find_consecutive_triple_sequence(NextValue, NextCards, Rest, [NextCards | Acc]);
find_consecutive_triple_sequence(_, _, Rest, Acc) ->
    {lists:flatten(lists:reverse(Acc)), Rest}.

%% 计算早期节奏
calculate_early_tempo(Play, Context) ->
    {Type, _, Cards} = Play,
    case Type of
        ?CARD_TYPE_SINGLE when Context#play_context.cards_remaining > 10 -> 0.3;
        ?CARD_TYPE_PAIR when Context#play_context.cards_remaining > 10 -> 0.5;
        ?CARD_TYPE_STRAIGHT -> 0.8;
        _ -> 0.6
    end.

%% 计算中期节奏
calculate_mid_tempo(Play, Context) ->
    {Type, _, Cards} = Play,
    case Type of
        ?CARD_TYPE_SINGLE when Context#play_context.cards_remaining > 5 -> 0.4;
        ?CARD_TYPE_PAIR when Context#play_context.cards_remaining > 5 -> 0.6;
        ?CARD_TYPE_STRAIGHT -> 0.7;
        _ -> 0.5
    end.

%% 计算后期节奏
calculate_end_tempo(Play, Context) ->
    {_, _, Cards} = Play,
    case length(Cards) of
        L when L =:= Context#play_context.cards_remaining -> 1.0;
        L when L > Context#play_context.cards_remaining / 2 -> 0.8;
        _ -> 0.6
    end.