-module(doudizhu_ai_strategy).

-export([
    evaluate_situation/2,
    choose_strategy/2,
    execute_strategy/3,
    calculate_move_score/3,
    analyze_hand_value/1
]).

-include("../include/card_types.hrl").
-include("../include/game_records.hrl").

-record(strategy_context, {
    game_stage,         % early_game | mid_game | end_game
    role,              % dizhu | nongmin
    hand_strength,     % 手牌强度评估
    control_level,     % 控制能力评估
    winning_prob,      % 胜率估计
    cards_remaining,   % 剩余手牌数
    opponent_cards     % 对手剩余牌数
}).

-record(hand_value, {
    singles = [],      % 单牌
    pairs = [],        % 对子
    triples = [],      % 三张
    sequences = [],    % 顺子
    bombs = [],        % 炸弹
    rockets = []       % 火箭
}).

%% 评估局势
evaluate_situation(GameState, PlayerState) ->
    HandCards = PlayerState#state.hand_cards,
    OpponentCards = get_opponent_cards(GameState),
    
    Context = #strategy_context{
        game_stage = determine_game_stage(GameState),
        role = PlayerState#state.role,
        hand_strength = evaluate_hand_strength(HandCards),
        control_level = calculate_control_level(GameState, PlayerState),
        winning_prob = estimate_winning_probability(GameState, PlayerState),
        cards_remaining = length(HandCards),
        opponent_cards = OpponentCards
    }.

%% 选择策略
choose_strategy(Context, Options) ->
    BaseStrategy = choose_base_strategy(Context),
    RefineStrategy = refine_strategy(BaseStrategy, Context, Options),
    adjust_strategy_for_endgame(RefineStrategy, Context).

%% 执行策略
execute_strategy(Strategy, GameState, PlayerState) ->
    HandCards = PlayerState#state.hand_cards,
    LastPlay = get_last_play(GameState),
    
    case Strategy of
        aggressive ->
            execute_aggressive_strategy(HandCards, LastPlay, GameState);
        conservative ->
            execute_conservative_strategy(HandCards, LastPlay, GameState);
        control ->
            execute_control_strategy(HandCards, LastPlay, GameState);
        explosive ->
            execute_explosive_strategy(HandCards, LastPlay, GameState)
    end.

%% 内部函数

%% 获取对手剩余牌数
get_opponent_cards(GameState) ->
    Players = GameState#game_state.players,
    lists:foldl(fun({_Pid, Cards, _Role}, Acc) -> 
        Acc + length(Cards)
    end, 0, Players).

%% 获取最后一手牌
get_last_play(GameState) ->
    case GameState#game_state.last_play of
        [] -> none;
        {_Pid, Cards} -> Cards
    end.

%% 确定游戏阶段
determine_game_stage(GameState) ->
    % 根据已出牌数量和总牌数判断游戏阶段
    PlayedCards = GameState#game_state.played_cards,
    TotalPlayed = lists:foldl(fun({_Pid, Cards}, Acc) -> 
        Acc + length(Cards)
    end, 0, PlayedCards),
    
    % 假设总牌数为54张
    case TotalPlayed of
        N when N < 18 -> early_game;
        N when N < 36 -> mid_game;
        _ -> end_game
    end.

%% 评估手牌强度
evaluate_hand_strength(HandCards) ->
    HandValue = analyze_hand_value(HandCards),
    
    % 计算各种牌型的权重得分
    SinglesScore = length(HandValue#hand_value.singles) * 0.1,
    PairsScore = length(HandValue#hand_value.pairs) * 0.2,
    TriplesScore = length(HandValue#hand_value.triples) * 0.3,
    SequencesScore = length(HandValue#hand_value.sequences) * 0.4,
    BombsScore = length(HandValue#hand_value.bombs) * 0.6,
    RocketsScore = length(HandValue#hand_value.rockets) * 1.0,
    
    % 归一化到0-1范围
    TotalScore = SinglesScore + PairsScore + TriplesScore + SequencesScore + BombsScore + RocketsScore,
    NormalizedScore = min(1.0, TotalScore / 10),
    NormalizedScore.

%% 计算控制水平
calculate_control_level(GameState, PlayerState) ->
    % 根据手牌中的高牌和炸弹数量评估控制能力
    HandCards = PlayerState#state.hand_cards,
    HandValue = analyze_hand_value(HandCards),
    
    % 计算高牌数量（2和大小王）
    HighCards = lists:filter(fun({Value, _}) -> 
        Value >= ?CARD_VALUE_2
    end, HandCards),
    
    % 计算控制得分
    HighCardScore = length(HighCards) * 0.2,
    BombScore = length(HandValue#hand_value.bombs) * 0.4,
    RocketScore = length(HandValue#hand_value.rockets) * 0.4,
    
    % 归一化到0-1范围
    ControlScore = HighCardScore + BombScore + RocketScore,
    min(1.0, ControlScore).

%% 估计胜率
estimate_winning_probability(GameState, PlayerState) ->
    % 简单估计：根据手牌数量和强度
    HandCards = PlayerState#state.hand_cards,
    HandStrength = evaluate_hand_strength(HandCards),
    CardsRemaining = length(HandCards),
    
    % 获取对手剩余牌数
    OpponentCards = get_opponent_cards(GameState),
    
    % 计算牌数优势
    CardAdvantage = case PlayerState#state.role of
        dizhu -> 
            % 地主初始多3张牌，所以需要调整
            case OpponentCards of
                0 -> 1.0; % 对手没牌了，必输
                _ -> max(0, 1.0 - (CardsRemaining / OpponentCards))
            end;
        nongmin -> 
            % 农民需要合作，所以计算方式不同
            case CardsRemaining of
                0 -> 1.0; % 自己没牌了，必赢
                _ -> max(0, 1.0 - (CardsRemaining / max(1, OpponentCards)))
            end
    end,
    
    % 综合手牌强度和牌数优势
    WinProb = HandStrength * 0.6 + CardAdvantage * 0.4,
    min(1.0, WinProb).

%% 选择基础策略
choose_base_strategy(Context) ->
    case {Context#strategy_context.game_stage, Context#strategy_context.role} of
        {early_game, dizhu} when Context#strategy_context.hand_strength >= 0.7 ->
            aggressive;
        {early_game, nongmin} when Context#strategy_context.hand_strength >= 0.8 ->
            control;
        {mid_game, _} when Context#strategy_context.winning_prob >= 0.7 ->
            control;
        {end_game, _} when Context#strategy_context.cards_remaining =< 5 ->
            explosive;
        _ ->
            conservative
    end.

%% 细化策略
refine_strategy(BaseStrategy, Context, Options) ->
    case {BaseStrategy, Context#strategy_context.control_level} of
        {conservative, Control} when Control >= 0.8 ->
            control;
        {aggressive, Control} when Control =< 0.3 ->
            conservative;
        {Strategy, _} ->
            Strategy
    end.

%% 调整终盘策略
adjust_strategy_for_endgame(Strategy, Context) ->
    case Context#strategy_context.cards_remaining of
        N when N =< 3 ->
            case Context#strategy_context.winning_prob of
                P when P >= 0.8 -> aggressive;
                P when P =< 0.2 -> explosive;
                _ -> Strategy
            end;
        _ ->
            Strategy
    end.

%% 执行进攻策略
execute_aggressive_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_aggressive_lead(HandValue);
        Play ->
            choose_aggressive_follow(HandValue, Play)
    end.

%% 执行保守策略
execute_conservative_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_conservative_lead(HandValue);
        Play ->
            choose_conservative_follow(HandValue, Play)
    end.

%% 执行控制策略
execute_control_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_control_lead(HandValue);
        Play ->
            choose_control_follow(HandValue, Play)
    end.

%% 执行爆发策略
execute_explosive_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_explosive_lead(HandValue);
        Play ->
            choose_explosive_follow(HandValue, Play)
    end.

%% 选择进攻性首出
choose_aggressive_lead(HandValue) ->
    case find_best_lead_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_single_card(HandValue)
    end.

%% 选择进攻性跟牌
choose_aggressive_follow(HandValue, LastPlay) ->
    case find_minimum_greater_combination(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> 
            case should_use_bomb(HandValue, LastPlay) of
                true -> find_smallest_bomb(HandValue);
                false -> pass
            end
    end.

%% 选择保守性首出
choose_conservative_lead(HandValue) ->
    case find_safe_lead(HandValue) of
        {ok, Move} -> Move;
        none -> find_smallest_single(HandValue)
    end.

%% 选择保守性跟牌
choose_conservative_follow(HandValue, LastPlay) ->
    case find_safe_follow(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> pass
    end.

%% 选择控制性首出
choose_control_lead(HandValue) ->
    case find_control_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_tempo_play(HandValue)
    end.

%% 选择控制性跟牌
choose_control_follow(HandValue, LastPlay) ->
    case should_maintain_control(HandValue, LastPlay) of
        true -> find_minimum_greater_combination(HandValue, LastPlay);
        false -> pass
    end.

%% 选择爆发性首出
choose_explosive_lead(HandValue) ->
    case find_strongest_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_any_playable(HandValue)
    end.

%% 选择爆发性跟牌
choose_explosive_follow(HandValue, LastPlay) ->
    case find_crushing_play(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> pass
    end.

%% 分析手牌价值
analyze_hand_value(Cards) ->
    GroupedCards = group_cards(Cards),
    #hand_value{
        singles = find_singles(GroupedCards),
        pairs = find_pairs(GroupedCards),
        triples = find_triples(GroupedCards),
        sequences = find_sequences(GroupedCards),
        bombs = find_bombs(GroupedCards),
        rockets = find_rockets(GroupedCards)
    }.

%% 计算出牌得分
calculate_move_score(Move, Context, LastPlay) ->
    BaseScore = calculate_base_score(Move),
    TempoScore = calculate_tempo_score(Move, Context),
    ControlScore = calculate_control_score(Move, Context),
    EfficiencyScore = calculate_efficiency_score(Move, Context),
    
    WeightedScore = BaseScore * 0.3 +
                    TempoScore * 0.2 +
                    ControlScore * 0.3 +
                    EfficiencyScore * 0.2,
    
    adjust_score_for_context(WeightedScore, Move, Context, LastPlay).

%% 辅助函数

group_cards(Cards) ->
    lists:foldl(fun(Card, Acc) ->
        {Value, _} = Card,
        maps:update_with(Value, fun(List) -> [Card|List] end, [Card], Acc)
    end, maps:new(), Cards).

find_singles(GroupedCards) ->
    maps:fold(fun(Value, [Card], Acc) ->
        [{Value, Card}|Acc];
        (_, _, Acc) -> Acc
    end, [], GroupedCards).

find_pairs(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 2 of
            true -> [{Value, lists:sublist(Cards, 2)}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_triples(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 3 of
            true -> [{Value, lists:sublist(Cards, 3)}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_bombs(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 4 of
            true -> [{Value, lists:sublist(Cards, 4)}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_rockets(GroupedCards) ->
    SmallJoker = maps:get(?CARD_VALUE_SMALL_JOKER, GroupedCards, []),
    BigJoker = maps:get(?CARD_VALUE_BIG_JOKER, GroupedCards, []),
    case {SmallJoker, BigJoker} of
        {[S|_], [B|_]} -> [{rocket, [S, B]}];
        _ -> []
    end.

find_sequences(GroupedCards) ->
    Values = lists:sort(maps:keys(GroupedCards)),
    find_straight_sequences(Values, GroupedCards, 5, []).

find_straight_sequences([], _, _, Acc) -> Acc;
find_straight_sequences([H|T], GroupedCards, MinLen, Acc) ->
    case find_sequence_from(H, T, GroupedCards, MinLen) of
        {ok, Seq, Rest} -> 
            Cards = [hd(maps:get(V, GroupedCards)) || V <- Seq],
            find_straight_sequences(Rest, GroupedCards, MinLen, [{H, Cards}|Acc]);
        {error, Rest} -> 
            find_straight_sequences(Rest, GroupedCards, MinLen, Acc)
    end.

find_sequence_from(Start, Values, GroupedCards, MinLen) ->
    find_sequence_from(Start, Values, GroupedCards, MinLen, [Start], 1).

find_sequence_from(_, [], _, MinLen, Seq, Len) when Len >= MinLen ->
    {ok, lists:reverse(Seq), []};
find_sequence_from(_, [], _, _, _, _) ->
    {error, []};
find_sequence_from(Prev, [H|T], GroupedCards, MinLen, Seq, Len) ->
    case H =:= Prev + 1 of
        true ->
            find_sequence_from(H, T, GroupedCards, MinLen, [H|Seq], Len+1);
        false when Len >= MinLen ->
            {ok, lists:reverse(Seq), [H|T]};
        false ->
            {error, [H|T]}
    end.

%% 查找最佳首出组合
find_best_lead_combination(HandValue) ->
    % 优先级：顺子 > 连对 > 三带一 > 对子 > 单牌
    case HandValue#hand_value.sequences of
        [{_, Cards}|_] -> {ok, Cards};
        [] ->
            case find_straight_pairs(HandValue) of
                {ok, Cards} -> {ok, Cards};
                none ->
                    case find_triple_with_single(HandValue) of
                        {ok, Cards} -> {ok, Cards};
                        none ->
                            case HandValue#hand_value.pairs of
                                [{_, Cards}|_] -> {ok, Cards};
                                [] ->
                                    case HandValue#hand_value.singles of
                                        [{_, Card}|_] -> {ok, [Card]};
                                        [] -> none
                                    end
                            end
                    end
            end
    end.

%% 查找连对
find_straight_pairs(HandValue) ->
    Pairs = HandValue#hand_value.pairs,
    case length(Pairs) >= 3 of
        true ->
            SortedPairs = lists:sort(fun({V1, _}, {V2, _}) -> V1 =< V2 end, Pairs),
            find_consecutive_pairs(SortedPairs, 3);
        false -> none
    end.

find_consecutive_pairs(Pairs, MinLen) ->
    find_consecutive_pairs(Pairs, MinLen, []).

find_consecutive_pairs([], _, []) -> none;
find_consecutive_pairs([], MinLen, Acc) when length(Acc) >= MinLen ->
    {ok, lists:flatten([Cards || {_, Cards} <- lists:reverse(Acc)])};
find_consecutive_pairs([], _, _) -> none;
find_consecutive_pairs([{V, Cards}|Rest], MinLen, []) ->
    find_consecutive_pairs(Rest, MinLen, [{V, Cards}]);
find_consecutive_pairs([{V, Cards}|Rest], MinLen, [{PrevV, _}|_]=Acc) ->
    case V =:= PrevV + 1 of
        true -> find_consecutive_pairs(Rest, MinLen, [{V, Cards}|Acc]);
        false when length(Acc) >= MinLen ->
            {ok, lists:flatten([C || {_, C} <- lists:reverse(Acc)])};
        false -> find_consecutive_pairs(Rest, MinLen, [{V, Cards}])
    end.

%% 查找三带一
find_triple_with_single(HandValue) ->
    case {HandValue#hand_value.triples, HandValue#hand_value.singles} of
        {[{TripleV, TripleCards}|_], [{SingleV, SingleCard}|_]} when TripleV =/= SingleV ->
            {ok, TripleCards ++ [SingleCard]};
        _ -> none
    end.

%% 查找单牌
find_single_card(HandValue) ->
    case HandValue#hand_value.singles of
        [{_, Card}|_] -> [Card];
        [] -> 
            case HandValue#hand_value.pairs of
                [{_, [Card|_]}|_] -> [Card];
                [] ->
                    case HandValue#hand_value.triples of
                        [{_, [Card|_]}|_] -> [Card];
                        [] -> []
                    end
            end
    end.

%% 查找最小单牌
find_smallest_single(HandValue) ->
    AllSingles = case HandValue#hand_value.singles of
        [] -> [];
        Singles -> [{V, C} || {V, C} <- Singles]
    end,
    
    case AllSingles of
        [] -> [];
        _ ->
            [{_, Card}|_] = lists:sort(fun({V1, _}, {V2, _}) -> V1 =< V2 end, AllSingles),
            [Card]
    end.

%% 查找最小炸弹
find_smallest_bomb(HandValue) ->
    case HandValue#hand_value.bombs of
        [] -> [];
        Bombs ->
            [{_, Cards}|_] = lists:sort(fun({V1, _}, {V2, _}) -> V1 =< V2 end, Bombs),
            Cards
    end.

%% 查找任意可出牌
find_any_playable(HandValue) ->
    case find_single_card(HandValue) of
        [] -> [];
        Cards -> Cards
    end.

%% 查找安全首出
find_safe_lead(HandValue) ->
    % 优先出对子和三张
    case HandValue#hand_value.pairs of
        [{_, Cards}|_] -> {ok, Cards};
        [] ->
            case HandValue#hand_value.triples of
                [{_, Cards}|_] -> {ok, Cards};
                [] -> none
            end
    end.

%% 查找安全跟牌
find_safe_follow(HandValue, LastPlay) ->
    % 简单实现：找到比LastPlay大的最小牌
    find_minimum_greater_combination(HandValue, LastPlay).

%% 查找控制组合
find_control_combination(HandValue) ->
    % 优先出单牌，保留炸弹和对子
    case HandValue#hand_value.singles of
        [{_, Card}|_] -> {ok, [Card]};
        [] -> find_safe_lead(HandValue)
    end.

%% 查找节奏出牌
find_tempo_play(HandValue) ->
    % 简单实现：出最小的单牌
    [Card] = find_smallest_single(HandValue),
    [Card].

%% 查找最小的大于LastPlay的组合
find_minimum_greater_combination(HandValue, LastPlay) ->
    % 简单实现，实际应该根据LastPlay的牌型查找对应的更大牌型
    % 这里假设LastPlay是单牌
    case LastPlay of
        [LastCard] ->
            {LastValue, _} = LastCard,
            GreaterSingles = [{V, C} || {V, C} <- HandValue#hand_value.singles, V > LastValue],
            case GreaterSingles of
                [] -> none;
                _ ->
                    [{_, Card}|_] = lists:sort(fun({V1, _}, {V2, _}) -> V1 =< V2 end, GreaterSingles),
                    {ok, [Card]}
            end;
        _ -> none % 简化处理，实际应该处理各种牌型
    end.

%% 查找压制性出牌
find_crushing_play(HandValue, _LastPlay) ->
    % 优先使用炸弹
    case HandValue#hand_value.bombs of
        [] -> 
            case HandValue#hand_value.rockets of
                [] -> none;
                [{_, Cards}|_] -> {ok, Cards}
            end;
        [{_, Cards}|_] -> {ok, Cards}
    end.

%% 查找最强组合
find_strongest_combination(HandValue) ->
    % 优先级：火箭 > 炸弹 > 顺子 > 三带 > 对子 > 单牌
    case HandValue#hand_value.rockets of
        [{_, Cards}|_] -> {ok, Cards};
        [] ->
            case HandValue#hand_value.bombs of
                [{_, Cards}|_] -> {ok, Cards};
                [] ->
                    case HandValue#hand_value.sequences of
                        [{_, Cards}|_] -> {ok, Cards};
                        [] ->
                            case HandValue#hand_value.triples of
                                [{_, Cards}|_] -> {ok, Cards};
                                [] ->
                                    case HandValue#hand_value.pairs of
                                        [{_, Cards}|_] -> {ok, Cards};
                                        [] ->
                                            case HandValue#hand_value.singles of
                                                [{_, Card}|_] -> {ok, [Card]};
                                                [] -> none
                                            end
                                    end
                            end
                    end
            end
    end.

%% 判断是否应该使用炸弹
should_use_bomb(HandValue, LastPlay) ->
    % 简单实现：当剩余牌数少于8张且有炸弹时考虑使用
    length(lists:flatten([C || {_, C} <- HandValue#hand_value.singles] ++ 
                         [C || {_, C} <- HandValue#hand_value.pairs] ++ 
                         [C || {_, C} <- HandValue#hand_value.triples])) < 8 andalso
    HandValue#hand_value.bombs =/= [].

%% 判断是否应该保持控制
should_maintain_control(HandValue, LastPlay) ->
    % 简单实现：当有足够的高牌时尝试保持控制
    HighCards = [{V, C} || {V, C} <- HandValue#hand_value.singles, V >= ?CARD_VALUE_J],
    length(HighCards) >= 2.

%% 计算基础得分
calculate_base_score(Move) ->
    % 简单实现：根据牌型和牌值计算基础得分
    case length(Move) of
        1 -> 0.3;
        2 -> 
            [{V1, _}, {V2, _}] = Move,
            case V1 =:= V2 of
                true -> 0.5; % 对子
                false -> 0.4 % 其他两张牌
            end;
        3 -> 0.6;
        4 -> 
            [{V1, _}, {V2, _}, {V3, _}, {V4, _}] = Move,
            case V1 =:= V2 andalso V2 =:= V3 andalso V3 =:= V4 of
                true -> 0.9; % 炸弹
                false -> 0.7 % 其他四张牌
            end;
        _ -> 0.8 % 长牌型
    end.

%% 计算节奏得分
calculate_tempo_score(Move, Context) ->
    % 简单实现：根据游戏阶段和出牌类型计算节奏得分
    case Context#strategy_context.game_stage of
        early_game -> 0.4;
        mid_game -> 0.6;
        end_game -> 0.8
    end.

%% 计算控制得分
calculate_control_score(Move, Context) ->
    % 简单实现：根据控制水平计算控制得分
    Context#strategy_context.control_level.

%% 计算效率得分
calculate_efficiency_score(Move, Context) ->
    % 简单实现：根据出牌数量和剩余牌数计算效率得分
    CardsPlayed = length(Move),
    CardsRemaining = Context#strategy_context.cards_remaining,
    min(1.0, CardsPlayed / max(1, CardsRemaining) * 3).

%% 根据上下文调整得分
adjust_score_for_context(Score, Move, Context, LastPlay) ->
    % 简单实现：根据游戏阶段和角色调整得分
    AdjustedScore = case Context#strategy_context.role of
        dizhu -> Score * 1.1;
        nongmin -> Score * 0.9
    end,
    
    % 根据游戏阶段进一步调整
    case Context#strategy_context.game_stage of
        early_game -> AdjustedScore * 0.9;
        mid_game -> AdjustedScore;
        end_game -> AdjustedScore * 1.1
    end.